<?php
 class ServiceReflection { private $_className = ''; private $_serviceClassReflection = null; public function __construct($className, $options=0) { $this -> _className = $className; $this -> reflectService($options); } public function getServiceClassReflection() { return $this -> _serviceClassReflection; } public function getClass() { return $this -> _className; } protected function reflectService($options=0) { $this -> _serviceClassReflection = ClassServiceReflection::createReflection(new ReflectionClass( $this -> getClass() ) ); } } abstract class BaseServiceReflection { private $_type = ''; private $_description = ''; public function __construct($type='', $description='') { $this -> setType($type); $this -> setDescription($description); } public function setType($value) { $this -> _type = (string)$value; } public function getType() { return $this -> _type; } public function setDescription($value) { $this -> _description = (string)$value; } public function getDescription() { return $this -> _description; } protected static function removeCommentElements($value) { $value = preg_replace('/(^[\\s]*\\/\\*\\*)|(^[\\s]\\*\\/)|(^[\\s]*\\*?\\s)|(^[\\s]*)|(^[\\t]*)/ixm', '', $value); $value = preg_replace('/(\*\/)$/i', '', $value); $value = trim($value); return $value; } protected static function prepareDocComment($value) { $value = self::removeCommentElements($value); $value = str_replace("\r", '', $value); $value = preg_replace("/([\\t])+/", "\t", $value); return explode("\n", $value); } protected static function getDescriptionBlock($value) { $commentLines = self::prepareDocComment($value); $result = ''; foreach($commentLines as $line) { if($line == '') continue; if($line{0} == '@') continue; $result .= $line; $result .= "\n"; } $result = trim($result); return $result; } } class NamedServiceReflectionBase extends BaseServiceReflection { private $_name = null; public function __construct($name='', $type='', $description='') { parent::__construct($type, $description); $this -> setName($name); } public function setName($value) { $this -> _name = (string)$value; } public function getName() { return $this -> _name; } } class ClassServiceReflection extends BaseServiceReflection { private $_methods = array(); public function setMethods($value) { if( !is_array($value) ) return; } public function getMethods() { return $this -> _methods; } public function getMethod($name) { if( !$this -> hasMethod($name) ) return null; return $this -> _methods[$name]; } public function hasMethod($name) { return isset($this -> _methods[$name]); } public function addMethod(MethodServiceReflection $value) { $this -> _methods[$value -> getName()] = $value; } static public function createReflection(ReflectionClass $reflection) { $result = new ClassServiceReflection($reflection -> getName(), self::getDescriptionBlock($reflection -> getDocComment())); $methods = $reflection -> getMethods(); foreach($methods as $method) { if( !$method -> isPublic()) continue; $result -> addMethod( MethodServiceReflection::createReflection($method) ); } return $result; } public function getTypes() { $result = array(); foreach($this -> getMethods() as $item) { $result = array_merge($result, $item -> getTypes()); } return $result; } } class MethodServiceReflection extends NamedServiceReflectionBase { private $_parameters = array(); private $_return = null; public function __construct($name='', $type='', $description='', $parameters=null, $return=null) { parent::__construct($name, $type, $description); $this -> setParameters($parameters); $this -> setReturn($return); } public function setParameters($value) { if( !is_array($value) ) return; $_parameters = array(); foreach($value as $param) { $this -> addParamter($param); } } public function getParameters() { return $this -> _parameters; } public function getNumberOfParameters() { return count($this -> _parameters); } public function addParamter(ParameterServiceReflection $value) { $this -> _parameters[$value -> getName()] = $value; } public function getParamter($name) { if( !isset($this -> _parameters[$name]) ) return null; return $this -> _parameters[$name]; } public function setReturn($value) { if( !$value instanceof ReturnServiceReflection) return; $this -> _return = $value; } public function getReturn() { return $this -> _return; } static public function createReflection(ReflectionMethod $reflection) { $comment = $reflection -> getDocComment(); $result = new MethodServiceReflection($reflection -> getName(), '', self::getDescriptionBlock($comment)); $commentLines = self::prepareDocComment($comment); $tmpParams = array(); $return = new ReturnServiceReflection('void', ''); $params = $reflection -> getParameters(); foreach($params as $param) { $name = $param -> getName(); $tmpParams[$name] = new ParameterServiceReflection($name, 'mixed'); if(version_compare(str_replace(PHP_EXTRA_VERSION, '', PHP_VERSION) , '5.3.16', 'eq')) continue; if( $param -> isDefaultValueAvailable() ) { $tmpParams[$name] -> setHasDefault(true); $tmpParams[$name] -> setRequired(false); $tmpParams[$name] -> setDefault( $param -> getDefaultValue(), false); } } foreach($commentLines as $line) { if ($line == '') continue; if ($line{0} != '@') continue; if( preg_match('/^@param\s+([\w\[\]()]+)\s+\$([\w()]+)\s*(.*)/i', $line, $match) ) { $type = $match[1]; $name = $match[2]; $desc = $match[3]; $param = new ParameterServiceReflection($name, $type, $desc); if( preg_match('/\(default=(.*)\)/i', $desc, $regs) ) { $param -> setHasDefault(true); $param -> setRequired(false); $default = $regs[1]; if($default == "''") { $param -> setDefault('', false); } elseif($default{0} == "'") { $default = substr($default, 1, strlen($default) -2); $param -> setDefault($default); } else { $param -> setDefault($default); } $desc = preg_replace('/' . preg_quote('(default=' . $regs[1] . ')', '/') . '/i', '', $desc); $param -> setDescription($desc); } $tmpParams[$name] = $param; } if( preg_match('/^@return\s+([\w\[\]()]+)\s*(.*)/i', $line, $match) ) { $return -> setType($match[1]); $return -> setDescription($match[2]); } if( preg_match('/^@result\s+([\w\[\]()]+)\s+\$([\w()]+)\s*(.*)/i', $line, $match) ) { $name = $match[2]; $type = $match[1]; $desc = $match[3]; if( preg_match('/\(default=(.*)\)/i', $desc, $regs) ) { $resultParam = new ParameterServiceReflection($name, $type, $desc); $resultParam -> setHasDefault(true); $resultParam -> setRequired(false); $default = $regs[1]; if($default == "''") { $resultParam -> setDefault('', false); } elseif($default{0} == "'") { $default = substr($default, 1, strlen($default) -2); $resultParam -> setDefault($default); } else { $resultParam -> setDefault($default); } $desc = preg_replace('/' . preg_quote('(default=' . $regs[1] . ')', '/') . '/i', '', $desc); $resultParam -> setDescription($desc); $return -> addResult( $resultParam ); } else { $return -> addResult( new NamedServiceReflectionBase($name, $type, $desc) ); } } } $result -> setParameters($tmpParams); $result -> setReturn($return); return $result; } public function getTypes() { $result = $this -> getReturn() -> getTypes(); foreach($this -> getParameters() as $item) { $result[$item -> getType()] = $item -> getType(); } return $result; } } class ParameterServiceReflection extends NamedServiceReflectionBase { private $_hasDefault = false; private $_required = false; private $_default = null; public function __construct($name='', $type='', $description='', $hasDefault = false, $required=true, $default=null) { parent::__construct($name, $type, $description); $this -> setDefault($default); $this -> setHasDefault($hasDefault); $this -> setRequired($required); } public function setDefault($value, $typeCast=true) { if($typeCast) { $type = gettype($value); switch( strToLower($type) ) { case 'boolean': case 'bool': $this -> _default = $value ? true : false; break; case 'double': case 'float': $this -> _default = (float)$value; break; case 'integer': $this -> _default = (integer)$value; break; case 'null': $this -> _default = null; break; case 'string': if($value == 'null') $this -> _default = null; elseif($value == 'true') $this -> _default = true; elseif($value == 'false') $this -> _default = false; else $this -> _default = $value; break; default: $this -> _default = $value; break; } } else { $this -> _default = $value; } $this -> setHasDefault(true); } public function getDefault() { return $this -> _default; } public function getDefaultAsText() { $result = $this -> _default; if($result === null) { $result = 'null'; } elseif( is_bool($result) ) { $result = $result ? 'true' : 'false'; } elseif( is_string($result) ) { if( is_numeric($result) OR $result === '' ) { } else { $result = '\'' . $result . '\''; } } return $result; } public function setRequired($value) { $this -> _required = (boolean)$value; } public function getRequired() { return $this -> _required; } public function setHasDefault($value) { $this -> _hasDefault = (boolean)$value; } public function getHasDefault() { return $this -> _hasDefault; } } class ReturnServiceReflection extends BaseServiceReflection { protected $_results = array(); public function getNumberOfResults() { return count($this -> _results); } public function setResults($value) { if(!is_array($value)) return; } public function getResults() { return $this -> _results; } public function addResult(NamedServiceReflectionBase $value) { $this -> _results[$value -> getName()] = $value; } public function getTypes() { $result = array(); $result[$this -> getType()] = $this -> getType(); foreach($this -> getResults() as $item) { $result[$item -> getType()] = $item -> getType(); } return $result; } } ?>